package com.delfines.business;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.List;

import com.delfines.Constante;
import com.delfines.dao.ConnectionFactory;
import com.delfines.vo.GttKardexVo;
import com.delfines.vo.Value;

import oracle.jdbc.OracleTypes;

public class ReporteKardexBo {
	
	public String getDate(){
		String fecha=""; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT TO_CHAR(SYSDATE, 'DD/MM/YYYY') hoy FROM DUAL");
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	fecha=rs.getString("hoy");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return fecha;
	}
	
	public List<Value> listarEstablecimiento(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
    		CallableStatement callableStatement = conn.prepareCall("{call SP_KOSTST(?)}");
    		callableStatement.registerOutParameter(1, OracleTypes.CURSOR);

    		callableStatement.executeUpdate();
    		rs = (ResultSet) callableStatement.getObject(1);
    		while (rs.next()) {
    			Value value=new Value();
    			value.setId(rs.getInt("KST_ID"));
    			value.setDenominacion(rs.getString("KST_NAME"));
    			
    			lista.add(value);
    		}
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        return lista;
	}
	
	
	public Integer createKardex(Integer vn_kst_id,Integer vn_art_id,String vs_tipo,String vd_fec_ini,String vd_fec_fin){
		Integer idKardex=Constante.ID_INICIAL; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
    		CallableStatement callableStatement = conn.prepareCall("{call MCDATA.SP_INVE_VALO_REAL(?,?,?,?,?,?)}");
    		callableStatement.setInt(1, vn_kst_id);
    		callableStatement.setInt(2, vn_art_id);
    		callableStatement.setString(3, vs_tipo);
    		callableStatement.setString(4, vd_fec_ini);
    		callableStatement.setString(5, vd_fec_fin);
    		callableStatement.registerOutParameter(6, Types.INTEGER);
    		
    		int i=callableStatement.executeUpdate();
    		
    		if(i>0){
    			idKardex=callableStatement.getInt(6);
    		}
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        return idKardex;
        
	}
	
	public List<GttKardexVo> reporteKardex(){
		List<GttKardexVo> lista = new ArrayList<GttKardexVo>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT ID_GTT_KARDEX, TO_CHAR(INSERT_DATE,'DD/MM/YYYY HH24:MI:SS') INSERT_DATE, QUANTITY_RECORDS, P_FECHA_INICIO,P_FECHA_FIN, KST_NAME ");
            query.append(",NVL(SUM_LAST_CST_TOT_SLD_FIN,0) SUM_LAST_CST_TOT_SLD_FIN,NVL(AJUSTE_LAST_CST_TOT_SLD_FIN,0) AJUSTE_LAST_CST_TOT_SLD_FIN  ");
            query.append(",NVL(SUM_LAST_CST_TOT_SLD_FIN,0)+NVL(AJUSTE_LAST_CST_TOT_SLD_FIN,0) LAST_CST_TOT_SLD_FIN  ");
            query.append(",KST_ID ");
            query.append("FROM ( ");
            query.append("SELECT X.ID_GTT_KARDEX, X.INSERT_DATE, X.QUANTITY_RECORDS, X.P_FECHA_INICIO, X.P_FECHA_FIN, K.KST_NAME, "); 
            query.append("X.SUM_LAST_CNT_SLD_FIN,X.SUM_LAST_CST_TOT_SLD_FIN,X.AJUSTE_LAST_CNT_SLD_FIN,X.AJUSTE_LAST_CST_TOT_SLD_FIN, ");
            query.append("K.KST_ID ");
            query.append("FROM MCDATA.GTT_KARDEX X ");   
            query.append("INNER JOIN MCDATA.KOSTST K ON (X.P_ID_ESTABLECIMIENTO=K.KST_ID) "); 
            query.append("WHERE X.STATUS_REG=1 ");
            query.append("ORDER BY X.ID_GTT_KARDEX DESC ");
            query.append(") ");
            
            ps = conn.prepareStatement(query.toString());
            rs = ps.executeQuery();
            
            while(rs.next()){
            	GttKardexVo consulta=new GttKardexVo(); 
            	consulta.setIdGttKardex(rs.getInt("ID_GTT_KARDEX"));
            	consulta.setFechaRegistro(rs.getString("INSERT_DATE"));
            	consulta.setCantidad(rs.getInt("QUANTITY_RECORDS"));
            	consulta.setFechaInicio(rs.getString("P_FECHA_INICIO"));
            	consulta.setFechaFin(rs.getString("P_FECHA_FIN"));
            	consulta.setEstablecimiento(rs.getString("KST_NAME"));
            	//--
            	consulta.setSumLastCstTotSldFin(rs.getDouble("SUM_LAST_CST_TOT_SLD_FIN"));
            	consulta.setAjusteLastCstTotSldFin(rs.getDouble("AJUSTE_LAST_CST_TOT_SLD_FIN"));
            	consulta.setLastCstTotSldFin(rs.getDouble("LAST_CST_TOT_SLD_FIN"));
            	//--
            	consulta.setIdKst(rs.getInt("KST_ID"));
            	
            	lista.add(consulta);
            }
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        return lista;
	}
	
	public void registrarAjuste(Integer idProceso,Double ajuste){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE MCDATA.GTT_KARDEX SET AJUSTE_LAST_CNT_SLD_FIN=0,AJUSTE_LAST_CST_TOT_SLD_FIN=? WHERE ID_GTT_KARDEX=?");
            
            ps = conn.prepareStatement(query.toString());
            ps.setDouble(1, ajuste);
            ps.setInt(2, idProceso);
            //--
            ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
	}
	
	public void eliminarKardex(Integer idKardex){
		PreparedStatement ps=null;
		ResultSet rs=null;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
    		CallableStatement callableStatement = conn.prepareCall("{call MCDATA.SP_DELETE_VALO_REAL(?)}");
    		callableStatement.setInt(1, idKardex);
    		
    		callableStatement.executeUpdate();
    		
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
            if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
	}
	
	
}