package com.delfines.spring.controller;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.OutputStream;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import com.delfines.business.InventarioDatcorpBo;
import com.delfines.business.ReporteKardexBo;
import com.delfines.dao.ConnectionFactory;
import com.delfines.dao.ConnectionFactorySQL;
import com.delfines.vo.GttKardexVo;
import com.delfines.vo.Inventario;

import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.export.JRPdfExporter;
import net.sf.jasperreports.engine.util.JRLoader;
import net.sf.jasperreports.export.SimpleExporterInput;
import net.sf.jasperreports.export.SimpleOutputStreamExporterOutput;

@Controller
public class DatcorpController {
	
	@Autowired
    private HttpServletRequest request;
	
    @RequestMapping(path={"/theurl"},method=RequestMethod.POST)
    public ModelAndView doUpload(@RequestParam("file") MultipartFile file,String fecha,String denominacion,String almacen,Model model,HttpServletRequest request) {
    	List<Inventario> lista=new LinkedList<>();
    	
    	if (!file.isEmpty()) {
             try {
            	String jasperDir = "/jasper/";
 	            String realPathtoJasper =  request.getServletContext().getRealPath(jasperDir);
 	            File dest = new File(realPathtoJasper+"/inventario.xls");
                file.transferTo(dest);
               
            	LoadDataFromExcelDatcorp load=new LoadDataFromExcelDatcorp(dest);
            	load.insertData(fecha,denominacion,almacen);
            	
            	InventarioDatcorpBo datcorp=new InventarioDatcorpBo();  
            	lista=datcorp.listarInventario();
            	
             }catch(Exception e){
            	 e.printStackTrace();
             }
    	}
    	
    	ModelAndView mv = new ModelAndView("datcorp-tabla");
    	mv.addObject("lista",lista);    	
   		return mv;
    }
    
	@RequestMapping(path={"/consulta-datcorp"}, method=RequestMethod.GET)
	public ModelAndView  consultaKardex(Model model,HttpServletRequest request) {
		InventarioDatcorpBo bo=new InventarioDatcorpBo();
	 	List<Inventario> lista=new ArrayList<Inventario>();
       	lista=bo.listarInventario();
	    
	    ModelAndView mv = new ModelAndView("datcorp-tabla");
	    mv.addObject("lista",lista);
        return mv;
	}
	
	@RequestMapping(path={"/impresion-reporte"}, method=RequestMethod.GET)
	public void  consultaReporteConsolidado(HttpServletRequest request, HttpServletResponse response) {
	    
		try {
			
    		String jasperDir = "/jasper/";
            String realPathtoJasper =  request.getServletContext().getRealPath(jasperDir);
            System.out.println(realPathtoJasper);
            
        	JasperReport reporte = (JasperReport) JRLoader.loadObject(new File(realPathtoJasper+"/report1.jasper"));

            Map<String, Object> parametros = new HashMap<String, Object>();
            
            parametros.put("P_IMAGE_PATH",realPathtoJasper+"/logo.png");
            
            Connection conn = ConnectionFactorySQL.getInstance().getConnection();
            
            JasperPrint jasperPrint = JasperFillManager.fillReport(reporte, parametros,conn);
            
            JRPdfExporter pdfExporter = new JRPdfExporter();
            pdfExporter.setExporterInput(new SimpleExporterInput(jasperPrint));
            ByteArrayOutputStream pdfReportStream = new ByteArrayOutputStream();
            pdfExporter.setExporterOutput(new SimpleOutputStreamExporterOutput(pdfReportStream));
            pdfExporter.exportReport();

            response.setContentType("application/pdf");
            response.setHeader("Content-Length", String.valueOf(pdfReportStream.size()));
            response.addHeader("Content-Disposition", "inline; filename=reporte_datcorp.pdf;");

            OutputStream responseOutputStream = response.getOutputStream();
            responseOutputStream.write(pdfReportStream.toByteArray());
            responseOutputStream.close();
            pdfReportStream.close();
        	
        } catch (Exception e) {
            e.printStackTrace();
        }
		
	}
	/**
	 * 
	 * @param fechaIni
	 * @param fechaFin
	 * @param idEstablecimiento
	 * @param model
	 * @param request
	 */
	@RequestMapping(path={"/create-datcorp"}, method=RequestMethod.GET)
	public void  createKardex(String fechaIni,String fechaFin,Integer idEstablecimiento,Model model,HttpServletRequest request) {
	    Thread thread = new Thread("Proceso Kardex "+System.currentTimeMillis()) {
	        public void run(){
	        	System.out.println("Inicio Proceso Kardex : "+getName());
	        	ReporteKardexBo bo=new ReporteKardexBo();
	        	Integer idKardex=bo.createKardex(idEstablecimiento,0,"00",fechaIni, fechaFin);
	        	System.out.println("Fin Proceso Kardex : "+getName()+" idKardex: "+idKardex);
	        }
	    };
	    thread.start();
	}
	
	
	
	@RequestMapping(path={"/parameter-datcorp"}, method=RequestMethod.GET)
	public void  parameterKardex(Integer idKardex,Integer idKst,Model model,HttpServletRequest request) {
		request.getSession().setAttribute("p_idKardex",idKardex);		
		request.getSession().setAttribute("p_idKst",idKst);
	}
	
	@RequestMapping(path={"/impresion-datcorp"}, method=RequestMethod.GET)
	public void  impresionKardex(HttpServletRequest request, HttpServletResponse response) {
	    try {
	    	
	    	Integer idKardex=(Integer)request.getSession().getAttribute("p_idKardex");
	    	Integer idKst=(Integer)request.getSession().getAttribute("p_idKst");
	    	
	    	
	    	if(idKardex!=null&&idKardex>0){
	    		String jasperDir = "/jasper/";
	            String realPathtoJasper =  request.getServletContext().getRealPath(jasperDir);
	            System.out.println(realPathtoJasper);
	            
	        	JasperReport reporte = (JasperReport) JRLoader.loadObject(new File(realPathtoJasper+"/reporte_kardex_detalle.jasper"));
	        	if(idKst.equals(3)){
	        		reporte = (JasperReport) JRLoader.loadObject(new File(realPathtoJasper+"/reporte_kardex_detalle_principal.jasper"));
	        	}
	        	
	            Map<String, Object> parametros = new HashMap<String, Object>();
	            parametros.put("P_IMAGE_PATH",realPathtoJasper+"/logo.jpg");
	            parametros.put("SUBREPORT_DIR",realPathtoJasper+"\\");
	            parametros.put("P_ID_GTT_KARDEX",idKardex);
	            
	            Connection conn = ConnectionFactory.getInstance().getConnection();
	            
	            JasperPrint jasperPrint = JasperFillManager.fillReport(reporte, parametros,conn);
	            
	            JRPdfExporter pdfExporter = new JRPdfExporter();
	            pdfExporter.setExporterInput(new SimpleExporterInput(jasperPrint));
	            ByteArrayOutputStream pdfReportStream = new ByteArrayOutputStream();
	            pdfExporter.setExporterOutput(new SimpleOutputStreamExporterOutput(pdfReportStream));
	            pdfExporter.exportReport();
	            
	            response.setContentType("application/pdf");
	            response.setHeader("Content-Length", String.valueOf(pdfReportStream.size()));
	            response.addHeader("Content-Disposition", "inline; filename=kardex.pdf;");
	            
	            OutputStream responseOutputStream = response.getOutputStream();
	            responseOutputStream.write(pdfReportStream.toByteArray());
	            responseOutputStream.close();
	            pdfReportStream.close();
	    	}
        } catch (Exception e) {
            e.printStackTrace();
        }
		
	}
	
	
    
   
    
}
